// Copyright (c) 2010 Moxie Marlinspike <moxie@thoughtcrime.org>

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 3 of the
// License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA


function Proxy() {
  this.name              = null;
  this.host              = null;
  this.interfaceLanguage = "en";
  this.searchLanguage    = "all";
  this.httpPort          = -1;
  this.sslPort           = -1;
  
  this.enabled           = false;
  this.filters           = new Array();

  this.proxyMaps         = true;
  this.proxyGroups       = true;
  this.proxyNews         = true;
  this.proxyVideo        = true;
  this.proxyProducts     = true;
  this.proxyImages       = true;
  this.proxyFinance      = true;

  this.useSSL            = true;

  this.setDefaultFilters();
}

Proxy.prototype.setDefaultFilters = function() {
  mainFilter = new Filter();
  mainFilter.setName("main");
  mainFilter.setExpression("^(?:(?!chatenabled\\.|mail\\.|checkout\\.|sites\\.|docs\\.|picasaweb\\.|notebook\\.|spreadsheets\\.|wave\\.|voice\\." +
			   (this.proxyMaps     ? "" : "|maps\\.|id\\.|mt[0-9]\\.|khm[0-9]\\.")     +
			   (this.proxyGroups   ? "" : "|groups\\.")   +
			   (this.proxyNews     ? "" : "|news\\.")     +
			   (this.proxyVideo    ? "" : "|video\\.")    +
			   (this.proxyProducts ? "" : "|products\\.") +
			   (this.proxyImages   ? "" : "|images\\.")   +
			   ")[^.]+\\.)?google\\.com(?!\\/accounts|\\/a\\/UniversalLogin|\\/calendar|\\/reader|\\/health|\\/notebook|\\/wave|\\/ig|\\/a\\/|\\/apps|\\/webmasters|\\/contacts|\\/voice" + 
			   (this.proxyProducts ? "" : "|\\/prdhp") +
			   (this.proxyImages   ? "" : "|\\/imghp")   +
			   ")");
  mainFilter.setEnabled(true);
   
  staticFilter  = new Filter();
  staticFilter.setName("static");
  staticFilter.setExpression(".+\\.gstatic\\.com/.?");
  staticFilter.setEnabled(true);

  hostedFilter  = new Filter();
  hostedFilter.setName("hosted");
  hostedFilter.setExpression(".+\\.googlehosted\\.com/.?");
  hostedFilter.setEnabled(true);

  hundredFilter = new Filter();
  hundredFilter.setName("1e100");
  hundredFilter.setExpression("(.+\\.)?1e100\\.net/.?");
  hundredFilter.setEnabled(true);
  
  filters = new Array();
  filters.push(mainFilter);
  filters.push(staticFilter);
  filters.push(hostedFilter);
  filters.push(hundredFilter);

  this.setFilters(filters);
};

Proxy.prototype.getHost = function() {
  return this.host;
};

Proxy.prototype.setHost = function(host) {
  this.host = host;
};

Proxy.prototype.getSSLPort = function() {
  return this.sslPort;
};

Proxy.prototype.setSSLPort = function(port) {
  this.sslPort = port;
};

Proxy.prototype.getHTTPPort = function() {
  return this.httpPort;
};

Proxy.prototype.setHTTPPort = function(port) {
  this.httpPort = port;
};

Proxy.prototype.getName = function() {
  return this.name;
};

Proxy.prototype.setName = function(name) {
  this.name = name;
};

Proxy.prototype.getFilters = function() {
  return this.filters;
};

Proxy.prototype.setFilters = function(filters) {
  this.filters = filters;
};

Proxy.prototype.getEnabled = function() {
  return this.enabled;
};

Proxy.prototype.setEnabled = function(value) {
  this.enabled = value;
};

Proxy.prototype.isSSLEnabled = function() {
  return this.useSSL;
}

Proxy.prototype.setSSLEnabled = function(value) {
  this.useSSL = value;
}

Proxy.prototype.setInterfaceLanguage = function(value) {
  this.interfaceLanguage = value;
};

Proxy.prototype.getInterfaceLanguage = function() {
  return this.interfaceLanguage;
};

Proxy.prototype.setSearchLanguage = function(value) {
  this.searchLanguage = value;
};

Proxy.prototype.getSearchLanguage = function() {
  return this.searchLanguage;
};

Proxy.prototype.getFilterCount = function() {
  return this.filters.length;
};

Proxy.prototype.setProxyMaps = function(value) {
  this.proxyMaps = value;
}

Proxy.prototype.getProxyMaps = function(value) {
  return this.proxyMaps;
}

Proxy.prototype.setProxyGroups = function(value) {
  this.proxyGroups = value;
}

Proxy.prototype.getProxyGroups = function(value) {
  return this.proxyGroups;
}

Proxy.prototype.setProxyNews = function(value) {
  this.proxyNews = value;
}

Proxy.prototype.getProxyNews = function(value) {
  return this.proxyNews;
}

Proxy.prototype.setProxyVideo = function(value) {
  this.proxyVideo = value;
}

Proxy.prototype.getProxyVideo = function(value) {
  return this.proxyVideo;
}

Proxy.prototype.setProxyProducts = function(value) {
  this.proxyProducts = value;
}

Proxy.prototype.getProxyProducts = function(value) {
  return this.proxyProducts;
}

Proxy.prototype.setProxyImages = function(value) {
  this.proxyImages = value;
}

Proxy.prototype.getProxyImages = function(value) {
  return this.proxyImages;
}

Proxy.prototype.setProxyFinance = function(value) {
  this.proxyFinance = value;
}

Proxy.prototype.getProxyFinance = function(value) {
  return this.proxyFinance;
}


Proxy.prototype.serialize = function(xmlDocument) {
  var proxyElement = xmlDocument.createElement("proxy");
  proxyElement.setAttribute("name", this.name);
  proxyElement.setAttribute("host", this.host);
  proxyElement.setAttribute("ssl-port", this.sslPort);
  proxyElement.setAttribute("http-port", this.httpPort);
  proxyElement.setAttribute("use-ssl", this.useSSL);
  proxyElement.setAttribute("enabled", this.enabled);
  proxyElement.setAttribute("interface-language", this.interfaceLanguage);
  proxyElement.setAttribute("search-language", this.searchLanguage);

  proxyElement.setAttribute("proxyMaps", this.proxyMaps);
  proxyElement.setAttribute("proxyGroups", this.proxyGroups);
  proxyElement.setAttribute("proxyNews", this.proxyNews);
  proxyElement.setAttribute("proxyVideo", this.proxyVideo);
  proxyElement.setAttribute("proxyProducts", this.proxyProducts);
  proxyElement.setAttribute("proxyImages", this.proxyImages);
  proxyElement.setAttribute("proxyFinance", this.proxyFinance);

//   var filtersElement = xmlDocument.createElement("filters");
//   proxyElement.appendChild(filtersElement);

//   for (key in this.filters) {
//     var filterElement = this.filters[key].serialize(xmlDocument);
//     filtersElement.appendChild(filterElement);
//   }

  return proxyElement;
};

Proxy.prototype.deserialize = function(element) {
  this.name              = element.getAttribute("name");
  this.host              = element.getAttribute("host");
  this.sslPort           = element.getAttribute("ssl-port");
  this.httpPort          = element.getAttribute("http-port");
  this.useSSL            = (element.getAttribute("use-ssl") == "true");
  this.interfaceLanguage = element.getAttribute("interface-language");
  this.searchLanguage    = element.getAttribute("search-language");
  this.enabled           = (element.getAttribute("enabled") == "true");
  this.proxyMaps         = (element.getAttribute("proxyMaps") == "true");
  this.proxyGroups       = (element.getAttribute("proxyGroups") == "true");
  this.proxyNews         = (element.getAttribute("proxyNews") == "true");
  this.proxyVideo        = (element.getAttribute("proxyVideo") == "true");
  this.proxyProducts     = (element.getAttribute("proxyProducts") == "true");
  this.proxyImages       = (element.getAttribute("proxyImages") == "true");
  this.proxyFinance      = (element.getAttribute("proxyFinance") == "true" || !element.getAttribute("proxyFinance"));
  this.setDefaultFilters();

//   var filterElements     = element.getElementsByTagName("filter");

//   if (filterElements.length > 0) {
//     this.filters = new Array()

//       for (var i=0;i<filterElements.length;i++) {
// 	var filterElement = filterElements.item(i);
// 	var filter        = new Filter();
// 	filter.deserialize(filterElement);
	
// 	this.filters.push(filter);
//       }
//   }

};

Proxy.prototype.matchesURL = function(url) {
  if (!this.enabled) return false;

  for (var i=0;i<this.filters.length;i++) {
    if (this.filters[i].matchesURL(url)) {
      return true;
    }
  }

  return false;
};